/*
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - June 7, 2002.
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2002-2007
   - HJ van Rantwijk.  All Rights Reserved.
   - 
   - Contributor(s):
   - 	Michael Vincent van Rantwijk <mv_van_rantwijk@yahoo.com>
   - 
*/

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const DEFAULT_ENABLED = true;
const DEFAULT_DELAY = 3000;
const DEFAULT_RETRY = 3000;
const DEFAULT_FREQUENCY = 0;

const INITIAL_DELAY = 1500;
const NEW_WINDOW_DELAY = 1000;

function debug(aMsg) {
  if (mzUpdateNotifier.jsConsoleMessages) {
    aMsg = ("mzUpdate Notifier: " + aMsg).replace(/\S{80}/g, "$&\n");
    Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService).logStringMessage(aMsg);
  }
}

/* :::::::: The Update Notifier Service ::::::::::::::: */

const mzUpdateNotifier = 
{
  initialized: 0,
  mTimer: null, // need to hold on to timer ref
  prefBranch : null,
  updateEnabled: DEFAULT_ENABLED,
  updateDelay: DEFAULT_DELAY,
  updateFrequency: DEFAULT_FREQUENCY,
  _nsIObserverService: null,
  jsConsoleMessages: false,

  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic) {
      case "app-startup":
        this._nsIObserverService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
        this._nsIObserverService.addObserver(this, "xpcom-shutdown", false);
        this._nsIObserverService.addObserver(this, "profile-after-change", false);
        break;

      case "profile-after-change":
        this._nsIObserverService.removeObserver(this, "profile-after-change", false);
        this._nsIObserverService.addObserver(this, "domwindowopened", false);
        break;

      case "domwindowopened":
        var self = this;
        var window = aSubject;
        window.addEventListener("load", 
          function() {
            self._onWindowOpened(window);
            window.removeEventListener("load", arguments.callee, false);
          }, 
          false);
        break;

      case "timer-callback":
        // dump("\nthis.initialized: " + this.initialized);
        switch(this.initialized) {
          case 0: 
            if (this.getMostRecentBrowserWindow())
              this.init();
            break;
          case 1: 
            this.checkForUpdate();
            break;
          case 2:
            this.initialized = 1;
            break;
        }
        break;

      case "nsPref:changed":
        switch(aData) {
          case "enabled":
            this.updateEnabled = true; // this._getPref(aData, this.updateEnabled);

            if (this.updateEnabled)
              this.mTimer.init(this, this.updateDelay, Ci.nsITimer.TYPE_ONE_SHOT);
            break;

          case "check-after-ms":
            this.updateDelay = this._getPref(aData, this.updateDelay);
            break;

          case "frequency":
            this.updateFrequency = this._getPref(aData, this.updateFrequency);
            // dump("\nthis.updateFrequency-2" + this.updateFrequency);
            break;
        }
        break;

      case "xpcom-shutdown":
        /* 
         * We need to drop our timer reference here to avoid a leak
         * since the timer keeps a reference to the observer! 
         */
        this.mTimer = null;
        this._nsIObserverService.removeObserver(this, "domwindowopened", false);
        this._nsIObserverService.removeObserver(this, "xpcom-shutdown", false);
        break;
    }
  },

  _onWindowOpened: function(aWindow)
  {
    if (aWindow.document.documentElement.getAttribute("windowtype") != "navigator:browser")
      return;

    if (this.initialized == 0) {
      this.mTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);

      this.prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("multizilla.updates.");
      this.prefBranch.QueryInterface(Ci.nsIPrefBranch2);

      this.updateEnabled = this._getPref("enabled", DEFAULT_ENABLED);
      this.updateDelay = this._getPref("check-after-ms", DEFAULT_DELAY);
      this.updateFrequency = this._getPref("frequency", DEFAULT_FREQUENCY); 
      this.checkInterval = (this.updateFrequency == 0) ? DEFAULT_DELAY : (this.updateFrequency * 24 * 60 * 1000);

      this.prefBranch.addObserver("enabled", this, false);
      this.prefBranch.addObserver("frequency", this, false);
      this.prefBranch.addObserver("check-after-ms", this, false);

      this.mTimer.init(this, INITIAL_DELAY, Ci.nsITimer.TYPE_ONE_SHOT);
    }
    else { // this.initialized is usually 1 here!
      var now = new Date().getTime();
      var nextCheck = new Date(this.nextUpdateCheck).getTime();

      if ((now + NEW_WINDOW_DELAY + 500) < nextCheck) {
         this.initialized = 2;
         this.mTimer.init(this, NEW_WINDOW_DELAY, Ci.nsITimer.TYPE_ONE_SHOT);
      }
    }
  },

  init: function()
  {
    // dump("\ninit() - 0 " + this.initialized);
    var checkDelay = 2500;
    var now = new Date().getTime();
    var browserWindow = this.getMostRecentBrowserWindow();

    if (browserWindow && 'getBrowser' in browserWindow) {
      this.initialized = 1;
      this.currentVersion = browserWindow.gBrowser.mVersion;
      this.currentRevision = browserWindow.gBrowser.mVersionRev;

      if (this.lastUpdateCheck > 0) {
        checkDelay = (now - this.lastUpdateCheck);

        if (checkDelay > this.checkInterval)
          checkDelay = 2500;
        else if(checkDelay < this.checkInterval)
          checkDelay = (this.checkInterval - checkDelay);
      }
      debug("Initialized with a " + checkDelay + "ms delay");
    }
    else {
      debug("Retry after a five second delay...");
      checkDelay = 5000;
    }
    this.mTimer.init(this, checkDelay, Ci.nsITimer.TYPE_ONE_SHOT);
  },

  checkForUpdate: function()
  {
    if (this.getOfflineStatus()) {
      debug("Status: off-line (automatic retry in 5 minutes)");
      this.mTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
      this.mTimer.init(this, DEFAULT_RETRY, Ci.nsITimer.TYPE_ONE_SHOT);
    }
    else {
      if (this.shouldCheckForUpdate()) {
        debug("Checking for updates...");
        try {
          var updateDatasourceURL = "http://multizilla.mozdev.org/xpi/updates.php";
          var currentVersion = this.currentVersion + "" + this.currentRevision;
          var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo)
          var sysInfo =  Components.classes["@mozilla.org/system-info;1"].getService(Components.interfaces.nsIPropertyBag2);
          var OSVersion = encodeURIComponent(sysInfo.getProperty("name") + " " +  sysInfo.getProperty("version"));
          // http://multizilla.mozdev.org/xpi/updates.php?multizilla=currentVersion&product=SeaMonkey&version=1.1.1&buildID=2007022206&OSVersion=Windows_NT%205.1
          var parameters = "?multizilla=" + currentVersion + "&product=" + appInfo.name + "&version=" + appInfo.version + "&buildID=" + appInfo.appBuildID + "&OSVersion=" + OSVersion;
          var xmlRequest = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
          // dump("\nupdateDatasourceURL: " + updateDatasourceURL + parameters);
          xmlRequest.open("GET", (updateDatasourceURL + parameters), true);
          // xmlRequest.channel.notificationCallbacks = new BadCertHandler();
          xmlRequest.setRequestHeader("Cache-Control", "no-cache");
          xmlRequest.onerror = function(aEvent)
          {
            // dump("\nonerror");
            var request, status;

            try {
              request = aEvent.target;
              // the following may throw (e.g. a local file or timeout)
              status = request.status;
            }
            catch(ex) {
              request = aEvent.target.channel.QueryInterface(Ci.nsIRequest);
              status = request.status;
            }
            // dump("\nstatus: " + status);
          }
          xmlRequest.onload = function()
          {
            // dump("\nonload");
            // if (xmlRequest.readyState == 4)
              // dump("\nxmlRequest.readyState == 4");
            // if (xmlRequest.status == 200)
              // dump("\nxmlRequest.status == 200");
          }
          xmlRequest.onprogress = function(aEvent)
          {
            // dump("\nonProgress: " + aEvent.position + "/" + aEvent.totalSize);
          }
          var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
          var ds = rdf.GetDataSource(updateDatasourceURL + parameters);
          ds = ds.QueryInterface(Ci.nsIRDFXMLSink);
          ds.addXMLSinkObserver(mzUpdateDatasourceObserver);
        }
        catch (ex) {
          debug("Exception getting updates.php: " + ex);
        }
      }
    }
  },

  shouldCheckForUpdate: function()
  {
    var shouldCheck = false;

    try {
      if (this._getPref("enabled", true)) {
        const lastCheckPrefID = "last_checked";
        // The update frequency pref is set to 0 by default
        // but other valid values are: 1, 7 and 30 (days)
        var freq = this._getPref("frequency", DEFAULT_FREQUENCY);
        var now = (new Date().valueOf()) / 1000; // milliseconds -> seconds
        var prefs = this.prefBranch;

        if (!prefs.prefHasUserValue(lastCheckPrefID))
          prefs.setIntPref(lastCheckPrefID, 0);
        var lastChecked = this._getPref(lastCheckPrefID, now);
        // dump("\nfreq: " + freq);
        if (freq > 0) {
          // dump("\n(lastChecked + freq) > now: " + (lastChecked + freq) > now);
          if ((lastChecked + freq) > now)
            return false;
        }
        prefs.setIntPref(lastCheckPrefID, now);
        prefs = Cc["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        prefs.savePrefFile(null);
        shouldCheck = true;
      }
    }
    catch (ex) {
      shouldCheck = false;
      debug("Exception in shouldCheckForUpdate: " + ex);
    }
    return shouldCheck;
  },

  getMostRecentBrowserWindow: function()
  {
    return this._getMostRecentWindow("navigator:browser");
  },

  getOfflineStatus: function()
  {
    return Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).offline;
  },

  _getMostRecentWindow: function(aWindowType)
  {
    return Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator).getMostRecentWindow(aWindowType);
  },

  _getPref: function(aName, aDefault) {
    var prefBranch = this.prefBranch;

    try {
      switch (prefBranch.getPrefType(aName))
      {
        case prefBranch.PREF_STRING:
             return prefBranch.getCharPref(aName);
        case prefBranch.PREF_BOOL:
             return prefBranch.getBoolPref(aName);
        case prefBranch.PREF_INT:
             return prefBranch.getIntPref(aName);
        default:
             return aDefault;
      }
    }
    catch(ex) {
      return aDefault;
    }
  },

  QueryInterface: function(aIID)
  {
    if (aIID.equals(Ci.nsIObserver) || 
        aIID.equals(Ci.nsIProfileStartupListener) ||
        aIID.equals(Ci.nsISupports))
          return this;
    Components.returnCode = Cr.NS_ERROR_NO_INTERFACE;
    return null;
  }
}

function checkCert(channel) {
  if (!channel.originalURI.schemeIs("https")) // bypass
    return;

  const Ci = Components.interfaces;  
  var cert = channel.securityInfo.QueryInterface(Ci.nsISSLStatusProvider).
      SSLStatus.QueryInterface(Ci.nsISSLStatus).serverCert;

  var issuer = cert.issuer;
  while (issuer && !cert.equals(issuer)) {
    cert = issuer;
    issuer = cert.issuer;
  }

  if (!issuer || issuer.tokenName != "Builtin Object Token")
    throw "cert issuer is not built-in";
}

function BadCertHandler() {
}
BadCertHandler.prototype = {

  // nsIBadCertListener
  confirmUnknownIssuer: function(socketInfo, cert, certAddType) {
    // dump("\nmzUpdateNotifier: BadCertHandler: Unknown issuer");
    return false;
  },

  confirmMismatchDomain: function(socketInfo, targetURL, cert) {
    // dump("\nmzUpdateNotifier: BadCertHandler: Mismatched domain");
    return false;
  },

  confirmCertExpired: function(socketInfo, cert) {
    // dump("\nmzUpdateNotifier: BadCertHandler: Expired certificate");
    return false;
  },

  notifyCrlNextupdate: function(socketInfo, targetURL, cert) {
  },

  // nsIChannelEventSink
  onChannelRedirect: function(oldChannel, newChannel, flags) {
    // make sure the certificate of the old channel checks out before we follow
    // a redirect from it.  See bug 340198.
    checkCert(oldChannel);
  },

  // nsIInterfaceRequestor
  getInterface: function(iid) {
    if (iid.equals(Components.interfaces.nsIBadCertListener) ||
        iid.equals(Components.interfaces.nsIChannelEventSink))
      return this;

    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
    return null;
  },

  // nsISupports
  QueryInterface: function(iid) {
    if (!iid.equals(Components.interfaces.nsIBadCertListener) &&
        !iid.equals(Components.interfaces.nsIChannelEventSink) &&
        !iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
        !iid.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

////////////////////////////////////////////////////////////////////////
//   mzUpdateDatasourceObserver : nsIRDFXMLSinkObserver
////////////////////////////////////////////////////////////////////////
var mzUpdateDatasourceObserver = 
{
  onBeginLoad: function(aSink) {},
  onInterrupt: function(aSink) {},
  onResume: function(aSink) {},

  onEndLoad: function(aSink)
  {
    aSink.removeXMLSinkObserver(this);
    var dataSource = aSink.QueryInterface(Ci.nsIRDFDataSource);
    var productName = this.getProductName();
    // dump("\nproductName: " + productName + "\n");
    var updateInfo = this.getUpdateInfo(dataSource, productName);
    mzUpdateNotifier.updateURL = updateInfo.URL;

    if (updateInfo) {
      var productObj = { name    : updateInfo.productName,
                         version : updateInfo.version,
                         revision: updateInfo.revision };
      var updateAvailable = this.newerVersionAvailable(productObj);
      // The following three lines restore the used version/revision 
      // info, but only when a browser window is available/opened!
      var browserWindow = this.getBrowserWindow();
      productObj.version = browserWindow.gBrowser.mVersion;
      productObj.revision = browserWindow.gBrowser.mVersionRev;

      if (updateAvailable) {
        debug("Update for: " + updateInfo.productName + " available!");
        // this.integrityCheck(updateInfo.URL, updateInfo.hash);
        var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
        var winWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher);
        var unBundle = this.getBundle("chrome://multiviews/locale/update-notifications.properties");

        if (!unBundle)
          return;

        var title = unBundle.formatStringFromName("updateDialogTitle", [updateInfo.productName], 1);
        var description = unBundle.formatStringFromName("updateDialogText", [updateInfo.productName, productObj.version, productObj.revision, updateInfo.productName, updateInfo.version, updateInfo.revision, updateInfo.productName], 7);
        var button0Text = unBundle.GetStringFromName("getItNow");
        var button1Text = unBundle.GetStringFromName("noThanks");
        var checkMsg = unBundle.GetStringFromName("dontAskAgain");
        var checkVal = {value:0};
        var result = promptService.confirmEx(winWatcher.activeWindow, title, description, 
                                             (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_IS_STRING) +
                                             (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_IS_STRING),
                                             button0Text, button1Text, null, checkMsg, checkVal);

        if (result == 0) {
          var ass = Cc["@mozilla.org/appshell/appShellService;1"].getService(Ci.nsIAppShellService);
          var installationText = updateInfo.text;
          var installationURL = updateInfo.URL;
          ass.hiddenDOMWindow.InstallTrigger.install({installationText : installationURL});
        }
        if (checkVal.value) {
          var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
          prefs.setBoolPref("multizilla.update_notifications.enabled", false);
        }
      }
      else {
        debug("No update for: " + updateInfo.productName + " available!");
      }
    }
  },

  onError: function(aSink, aStatus, aErrorMsg)
  {
    // dump("\naStatus: " + aStatus);
    if (aStatus == 2152398918)
      debug("Failed (off-line)");
    else if (aStatus == 2152398878)
      debug("Failed (no connection)");
    else
      debug("Error: " + aStatus + ": " + aErrorMsg);
    aSink.removeXMLSinkObserver(this);
  },

  getBrowserWindow: function()
  {
    var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
    return windowManager.getMostRecentWindow("navigator:browser");
  },

  getBundle: function(aURL)
  {
    var strBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
    return strBundleService.createBundle(aURL);
  },

  getProductName: function()
  {
    var browserWindow = this.getBrowserWindow();
    var tabBrowser = (browserWindow) ? browserWindow.getBrowser() : null;

    return (tabBrowser && 'mProduct' in tabBrowser) ? tabBrowser.mProduct : undefined;
  },

  getUpdateInfo: function(aDS, aProductName)
  {
    var updateInfo = new Object;

    try {
      var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
      var src = "urn:updates:latest";
      updateInfo.registryName = this._getTarget(rdf, aDS, src, "registryName");
      updateInfo.productName  = this._getTarget(rdf, aDS, src, "productName");

      if (aProductName == undefined || aProductName == "undetermined")
        aProductName = "Win";
      else
        aProductName += "-";

      updateInfo.version = this._getTarget(rdf, aDS, src, aProductName + "Version");
      updateInfo.revision = this._getTarget(rdf, aDS, src, aProductName + "Revision");
      updateInfo.text = this._getTarget(rdf, aDS, src, aProductName + "Text");
      updateInfo.URL = this._getTarget(rdf, aDS, src, aProductName + "URL");
      updateInfo.hash = this._getTarget(rdf, aDS, src, aProductName + "Hash");
    }
    catch (ex) {
      debug("Exception getting update info: " + ex);
      updateInfo = null;
    }
    return updateInfo;
  },

  newerVersionAvailable: function(aProductObj)
  {
    var browserWindow = this.getBrowserWindow();
    
    if (!browserWindow)
      return false;

    var vc = Cc["@mozilla.org/xpcom/version-comparator;1"].getService(Ci.nsIVersionComparator);

    if (aProductObj.name == "MultiZilla") {
      if (vc.compare(String(browserWindow.gBrowser.mVersion + browserWindow.gBrowser.mVersionRev), 
                     String(aProductObj.version + aProductObj.revision)) == -1)
        return true;
    }
    /* else if (aProductObj.name == "GoogleBox") {
      var mzToolbox = browserWindow.document.getElementById("MultiZillaToolbar");

      if (mzToolbox.mToolboxActivated) { // GoogleBox installed?
        var googleBox = browserWindow.document.getElementById("internalToolBox");
        // Pre GoogleBox v1.4a versions had no version info, that's why we check for 'undefined'
        var googleBoxVersion = (googleBox.mVersion == undefined) ? 0 : googleBox.mVersion;
        var googleBoxRevision = (googleBox.mVersionRev == undefined) ? "" : googleBox.mVersionRev;

        if (vc.compare(String(googleBoxVersion + googleBoxRevision), 
                       String(aProductObj.version + aProductObj.revision)) == -1)
          return true;
      }
      return false;
    } */
  },

  integrityCheck: function(aURL, aHash)
  {
    var _nsIIOService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
    var channel = _nsIIOService.newChannel(aURL, 0, null);
    var listener = {
      _buffer: [],
      _stream: null,
      _countRead: 0,

      binaryToHex: function(input)
      {
        var result = "";

        for (var i = 0; i < input.length; ++i) {
          var hex = input.charCodeAt(i).toString(16);

          if (hex.length == 1)
            hex = "0" + hex;
          result += hex;
        }
        return result;

        /* return [('0' + input.charCodeAt(i).toString(16)).slice(-2)
                for (i in input)].join(''); */
      },

      onStartRequest: function(aRequest, aContext) {
        this._stream = Cc["@mozilla.org/binaryinputstream;1"].createInstance(Ci.nsIBinaryInputStream);
      },
      onDataAvailable: function(aRequest, aContext, aInputStream, aOffset, aCount) {
        this._stream.setInputStream(aInputStream);
        this._buffer = this._buffer.concat(this._stream.readByteArray(aCount));
        this._countRead += aCount;
      },
      onStopRequest: function(aRequest, aContext, aStatusCode) {
        aRequest = aRequest.QueryInterface(Ci.nsIHttpChannel);
        var hashCode = null;

        if (Components.isSuccessCode(aStatusCode)) {
          // dump("\nChecking integrity...");
          var hashFunction = Ci.nsICryptoHash["SHA512"];
          var hash = Cc["@mozilla.org/security/hash;1"].createInstance(Ci.nsICryptoHash);
          hash.init(hashFunction);
          hash.update(this._buffer, this._buffer.length);
          hashCode = this.binaryToHex(hash.finish(false));
          dump("\nPackage Hash Code: " + hashCode);
        }
        this.buffer = new Array();
        return (hashCode == aHash);
      }
    }
    channel.asyncOpen(listener, null);
  },

  _getTarget: function(aRDF, aDS, aSrc, aProp)
  {
    var src = aRDF.GetResource(aSrc);
    var arc = aRDF.GetResource("http://home.netscape.com/NC-rdf#" + aProp);
    var target = aDS.GetTarget(src, arc, true);
    return target.QueryInterface(Ci.nsIRDFLiteral).Value;
  }
}

/* :::::::: Service Registration & Initialization ::::::::::::::: */

/* ........ nsIModule .............. */

var mzUpdateNotifierModule = 
{
  mClassName: "MultiZilla Update Notifier Service",
  mContractID: "@multizilla.org/update-service;1",
  mClassID: Components.ID("9e95b5cc-9387-461f-bf36-2708ce28bc48"),

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aCID.equals(this.mClassID))
      throw Cr.NS_ERROR_NO_INTERFACE;
    if (!aIID.equals(Ci.nsIFactory))
      throw Cr.NS_ERROR_NOT_IMPLEMENTED;

    return this.mFactory;
  },

  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    debug("Registering...\n");
    aCompMgr = aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(this.mClassID, this.mClassName, this.mContractID, aFileSpec, aLocation, aType);
    // this.getCategoryManager().addCategoryEntry("profile-startup-category", this.mContractID, "", true, true);

    // Make the Web Feed Service a startup observer
    this.getCategoryManager().addCategoryEntry("app-startup", this.mClassName,
                                               "service," + this.mContractID, 
                                               true, true, null);
  },

  unregisterSelf: function(aCompMgr, aFileSpec, aLocation)
  {
    debug("Unregistering...\n");
    aCompMgr = aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(this.mClassID, aFileSpec);
    this.getCategoryManager().deleteCategoryEntry("app-startup", this.mContractID, true);
  },

  canUnload: function(aCompMgr)
  {
    return true;
  },

  getCategoryManager: function()
  {
    return Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
  },

  /* ........ nsIFactory .............. */

  mFactory:
  {
    createInstance: function(aOuter, aIID)
    {
      if (aOuter != null)
        throw Cr.NS_ERROR_NO_AGGREGATION;
      if (!aIID.equals(Ci.nsIObserver) &&
          !aIID.equals(Ci.nsIProfileStartupListener) &&
          !aIID.equals(Ci.nsISupports))
        throw Cr.NS_ERROR_INVALID_ARG;

      return mzUpdateNotifier.QueryInterface(aIID);
    },

    lockFactory: function(aLock)
    {
      // quiten warnings
    }
  }
};

function NSGetModule(aCompMgr, aFileSpec)
{
  return mzUpdateNotifierModule;
}


